use std::collections::HashMap;

use serde::Deserialize;

#[derive(Debug, Clone, Deserialize)]
pub struct Config {
    pub projects: HashMap<String, Project>,
}

#[derive(Debug, Clone, Deserialize)]
pub struct Project {
    pub name: String,
    pub description: String,
    pub website: Option<String>,
    pub documentation: Option<String>,
    pub since: Option<String>,
    pub contact: Option<ContactInfo>,
    pub sub: Option<HashMap<String, Project>>,
}

#[derive(Debug, Clone, Deserialize)]
pub struct ContactInfo {
    pub email: Option<String>,
}

impl ContactInfo {
    pub fn build(&self) -> maud::PreEscaped<String> {
        maud::html!(
            @if let Some(mail) = &self.email {
                h3 { "Contact:" };
                p { "Mail: "; a href=(format!("mailto:{mail}")) { (mail) }};
            }
        )
    }
}

impl Config {
    pub fn load(path: &str) -> Self {
        let content = std::fs::read_to_string(path).unwrap();
        serde_yml::from_str(&content).unwrap()
    }
}

impl Project {
    pub fn build(&self, card_id: &str) -> maud::PreEscaped<String> {
        let subcard_id = format!("{card_id}_sub");

        maud::html!(
            div class="card" id=(card_id) {
                h3 { (self.name) };
                p { (self.description) };
                @if let Some(website) = &self.website {
                    p { "Website: "; a href=(website) { (website) }};
                }
                @if let Some(doc) = &self.documentation {
                    p { "Documentation: "; a href=(doc) { (doc) }};
                }
                @if let Some(since) = &self.since {
                    p { "Started: "; a href=(since) { (since) }};
                }
                @if let Some(contact) = &self.contact {
                    (contact.build())
                }
                @if self.sub.as_ref().map(|x| !x.is_empty()).unwrap_or(false) {
                    button class="expand-button" onclick=(format!("toggleSubcards('{subcard_id}')")) { "Expand" };
                }
                div class="subcards" id=(subcard_id) {
                    @if let Some(sub) = &self.sub {
                    @for (id, prj) in sub {
                        (prj.build(id))
                    }
                    }
                };
            };
        )
    }
}
